@@ -25,7 +25,7 @@ lazy_static! {
         .filter(|&w| {
             (w.len() > 3 || VALID_SHORT.contains(w))
                 && !is_blacklisted(w)
-                && !is_ignore_fp(w.chars())
+                && is_ignore_fp(w.chars()) == 0
         })
         .collect();
     static ref PROFANITY: Vec<&'static str> = include_str!("profanity.csv")
@@ -48,15 +48,23 @@ lazy_static! {
         .collect();
 }
 
-pub fn is_ignore_fp<C: Iterator<Item = char>>(text: C) -> bool {
-    Censor::new(text)
-        .with_ignore_false_positives(true)
+pub fn is_ignore_fp<C: Iterator<Item = char>>(text: C) -> usize {
+    let mut censor = Censor::new(text);
+    censor.with_ignore_false_positives(true);
+
+    if censor
         .analyze()
         .is(Type::PROFANE | Type::OFFENSIVE | Type::SEXUAL | Type::MEAN)
+    {
+        censor.total_matches().max(1)
+    } else {
+        0
+    }
 }
 
 fn maybe_false_positive<C: Iterator<Item = char> + Clone>(word: C) -> Option<String> {
-    if is_ignore_fp(word.clone()) {
+    let baseline = is_ignore_fp(word.clone());
+    if baseline > 0 {
         let word: String = word.collect();
         let word = &word[..];
         if is_blacklisted(word) {
@@ -87,7 +95,8 @@ fn maybe_false_positive<C: Iterator<Item = char> + Clone>(word: C) -> Option<Str
                 DICTIONARY.contains(sub_slice)
             };
 
-            if valid && is_ignore_fp(sub_slice.chars()) && !is_blacklisted(sub_slice) {
+            let subslice_matches = is_ignore_fp(sub_slice.chars());
+            if valid && subslice_matches >= baseline && !is_blacklisted(sub_slice) {
                 shortest_subslice = sub_slice;
             }
         }
